class DcTunnel
  require 'net/http'
  require 'advanced_hash_keys'

  def initialize(settings)
    @settings = settings
  end

  def get_settings
    @settings
  end

  def logger(content)
    unless self.get_settings[:silence]
      puts content
    end
  end

  def res_handler(res)
    integer_code = res.code.to_i
    if integer_code >= 500
      {
          error_code: integer_code,
          error_message: '服务器异常'
      }
    else
      res_body = JSON.parse(res.body)
      if integer_code >=200 && integer_code < 400
        res_body.symbolize_keys(true)
      elsif integer_code >= 400 && integer_code < 500
        {
            error_code:integer_code,
            error_message: res_body['message'] || res.message
        }
      end
    end
  end

  def http_get(url)
    self.logger "--- Tunnel : start get #{url} "
    res = Net::HTTP.get_response(URI(url))
    res = self.res_handler res
    self.logger "--- Tunnel : got symbolize_keys! response #{res} "
    res
  end

  def http_post(url, postdata={})
    self.logger "--- Tunnel : start get #{url} "
    res = Net::HTTP.post_form(URI(url), postdata)
    res = self.res_handler res
    self.logger "--- Tunnel : got response #{res} "
    res
  end

  def sso_path(path, params = {})
    settings = self.get_settings
    self.logger "--- Tunnel : sso_path set to #{settings[:sso_host]} "
    auth = {
        system_id: settings[:system_id],
        system_token: settings[:system_token]
    }
    self.logger "--- Auth : #{auth}"
    "#{settings[:sso_host]}#{path}.json?#{params.to_param}&#{auth.to_param}"
  end

  def get_user(id)
    self.http_get self.sso_path("/users/#{id}")
  end

  def get_factories(params = {})
    self.http_get self.sso_path("/factories", params)
  end

  def get_departments(params = {})
    self.http_get self.sso_path('/departments', params)
  end

  def get_users(params = {})
    # available params:
    # id phone department_ids factory_ids
    self.http_get self.sso_path("/users", params)
  end

  def get_locations(params={})
    self.http_get self.sso_path("/locations", params)
  end

  def get_validate_code(params = {})
    self.http_get self.sso_path('/users/get_validate_code',params)
  end

  def login(phone,password)
    self.http_post self.sso_path('/users/login'),{phone:phone,password:password}
  end

  def logout(sso_id)
    self.http_post self.sso_path("/users/#{sso_id}/logout",{})
  end

  def synchronize_users
    res = self.http_get self.sso_path('/users', {:system_ids => [self.get_settings[:system_id]]})
    allowed_ids = []
    res[:users].each do |sso_user|
      self.logger "sso_user is #{sso_user}"
      self.logger "sso_user_id is #{sso_user[:id]}"
      user = User.find_by_sso_id sso_user[:id]
      self.logger "user is #{user}"
      allowed_ids.push sso_user[:id]
      if user.nil?
        create_attributes = {
            sso_id: sso_user[:id]
        }
        get_settings[:default_user_props].each do |item|
          name = item[0]
          value = item[1]
          create_attributes[name] = sso_user[name].present? ? sso_user[name] : value
        end
        self.logger "create_attributes is #{create_attributes}"
        User.create create_attributes
      else
        update_attributes = {
            name: sso_user[:name],
            phone: sso_user[:phone],
            sso_id: sso_user[:id]
        }
        self.logger "update_attributes is #{update_attributes}"
        user.update(update_attributes)
      end
    end
    # 子系统内同步删除
    self.logger "allowed sso ids is #{allowed_ids}"
    User.where.not(:sso_id => allowed_ids).destroy_all
    User.where(:sso_id => nil).destroy_all
  end


  # tmp
  def sync_token(params)
    self.http_post self.sso_path("/users/sync_token",params)
  end
end